/* Copyright (c) 1998, 2002, Oracle Corporation.  All rights reserved.  */
/* 
   NAME 
     pvm.h - PL/SQL Virtual Machine micro-kernel implementations.

     **!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     **!!!! THIS FILE IS SHIPPED FOR NCOMP.                        !!!!
     **!!!!                                                        !!!!
     **!!!! If you change it for a bug fix, you will need to make  !!!!
     **!!!! sure it is re-shipped also along with the new binaries.!!!!
     **!!!! Please make this note in the BUGDB along with your fix.!!!!
     **!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   DESCRIPTION 
     PL/SQL runtime kernels. Implementation of most instructions.
     Contains runtime libraries/interfaces used by:
       -- ncomp generated C code
       -- native run-time libraries (pen.c, pvm.c, etc.)
       -- interpreted run-time (pfrrun.c/pvm.c, etc.).

     CAUTION:

     Do not put any interpreter specific definitions in pen.h (or files
     included by pen.h such as pvm.h/pdtyp.h/pptyp.h). Such definitions
     belong in pfrdef.h/pfmdef.h. Internal support functions not required
     by generated C code belongs to pet.h/pvm0.h.

   RELATED DOCUMENTS 
 
   EXPORT FUNCTION(S) 

   INTERNAL FUNCTION(S)

   EXAMPLES

   NOTES
     All routines in this module return a PVM processor status.
     Unless the instruction is a comparison which sets the
     processor status word, the return status will be
      PW_NONE - success
      PW_NUL  - failure. In this case percerr has the actual error
                code.

   MODIFIED   (MM/DD/YY)
   mvemulap    01/16/02 - bug fix for 1827146
   kmuthukk    12/20/01 - remove pevm_MOVL, pevm_MOVCADT, pevm_MOVS
   cbarclay    09/26/01 - pevm_CMPIO
   sagrawal    09/07/01 - array of record in binds
   sagrawal    08/31/01 - bulk define of record collections
   sylin       09/20/01 - Modify prototype for pevm_PIPE()
   cbarclay    08/07/01 - treat
   cwethere    08/09/01 - Add ABSN.
   dbronnik    09/07/01 - Add pevm_CONCN.
   cwethere    08/27/01 - Add instructions.
   sagrawal    05/07/01 - pevm_DRCAL
   dbronnik    04/17/01 - add flags to i4exim
   mvemulap    04/08/01 - fix prototype for pevm_INBI_INDEXED_UROWID
   mvemulap    04/10/01 - bug fix for 1712683
   kmuthukk    02/02/01 - move pevm_BSETN to pfrrun.c
   kmuthukk    01/26/01 - fast reinit pkgs
   mvemulap    03/29/01 - compiler warnings
   mvemulap    03/07/01 - fix native compiler warnings
   mvemulap    02/28/01 - remove while(0) to reduce compile time
   mvemulap    02/12/01 - olint fixes
   mxyang      02/20/01 - remove pevm_MOVCT
   mxyang      02/09/01 - CMP3LOB
   kmuthukk    12/21/00 - remove obsolete COPM instruction
   kmuthukk    12/19/00 - INCI/DECI support
   kmuthukk    12/12/00 - optimize pevm_INSI_SSCALAR
   gviswana    12/01/00 - ub4 for line numbers
   kmuthukk    12/01/00 - remove s.h include
   kmuthukk    12/01/00 - pvm.h cannot refer to ped.h functions
   dbronnik    11/29/00 - add pvm_ctx_pub
   wxli        12/01/00 - remove pvm_cnschk() from this file
   mvemulap    11/02/00 - change pevm_SNCAL
   mvemulap    10/30/00 - move lnr to perc
   mvemulap    10/20/00 - add arg_block to pevm_RCAL
   mvemulap    10/16/00 -  
   mvemulap    10/05/00 - add HS to pevm_ENTER
   jmuller     10/23/00 - Fix bug 1089498: publish pvm_cnschk()
   mvemulap    09/28/00 -  
   wxli        09/07/00 - pevm_CCNST with ctx
   mvemulap    09/14/00 -  
   mvemulap    09/07/00 - change GRWFCC
   wxli        08/25/00 - set lensem into CHINFO
   mvemulap    08/24/00 - lrg
   mvemulap    08/14/00 -  
   mvemulap    06/21/00 - more microkernels
   mvemulap    06/19/00 - native exec
   mvemulap    05/19/00 - pvm -> pevm
   wxli        07/17/00 - char/nchar implicit conversion
   mvemulap    07/09/00 - nested coll
   asethi      07/12/00 - Bulk bind extensions
   sagrawal    07/03/00 - Dynamic dispatch
   kmuthukk    05/27/00 - newrep: remove pvm_MOVFADT
   kmuthukk    04/26/00 - change INRDH prototyp
   asethi      06/13/00 - Added code to process PIPE statement
   wxli        06/11/00 - length semantics implementation
   mvemulap    05/16/00 - 
   mvemulap    04/24/00 - plsql ncomp
   mvemulap    04/19/00 - more micros
   mvemulap    04/08/00 - more micro kernels
   mvemulap    02/23/00 - Microkernels for PL/SQL VM
   mvemulap    02/23/00 - Creation

*/

#ifndef PVM_ORACLE
# define PVM_ORACLE

#ifndef PDTYP_ORACLE
# include <pdtyp.h>
#endif

/*---------------------------------------------------------------------------
                     PUBLIC TYPES AND CONSTANTS
 ---------------------------------------------------------------------------*/

/* public part of the run-time context structure */
struct pvm_ctx_pub
{
  ub4 ctlc_cnt;
};
typedef struct pvm_ctx_pub pvm_ctx_pub;

/*---------------------------------------------------------------------------
                     PRIVATE TYPES AND CONSTANTS
 ---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------
                           PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/


/* Bad instructions */
pevm_excs pevm_BAD(void *ctx);

/* The following branch instructions are directly implemented, 
 * so should never execute them through PVM
 */
#define pevm_BRNCH  pevm_BAD
#define pevm_BRGE   pevm_BAD
#define pevm_BRGT   pevm_BAD
#define pevm_BRLE   pevm_BAD
#define pevm_BRNE   pevm_BAD
#define pevm_BRNEG  pevm_BAD
#define pevm_BRNUL  pevm_BAD
#define pevm_BRZER  pevm_BAD

/*
 * BRREINI (Special Branch): used for fast package reinitialization.
 */
boolean pevm_BRREINI(void *ctx);

pevm_excs pevm_ABSI(void *ctx, void const *src1, void *dst);
pevm_excs pevm_ADDI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_ADDN(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_BNDUC(void *ctx, void const *src1, ub2 position, ub2 tmpub2, ub2
              flags);
pevm_excs pevm_BREAK(void *ctx);
pevm_excs pevm_BFTCHC(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CLREX(void *ctx);
pevm_excs pevm_RASRX(void *ctx, boolean native_exec);
pevm_excs pevm_CEXCP(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CSEXCP(void *ctx, void *dst);
pevm_excs pevm_CBEG(void *ctx, ub1 subopc, 
                    void const *src1, void *dst, void const *src2);
pevm_excs pevm_CSBEG(void *ctx, ub1 subopc, void *dst, void const *src1);
pevm_excs pevm_CMP3C(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3D(void *ctx, ub1 opc, void const *src1, void const *src2);
pevm_excs pevm_CMP3I(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3N(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3R(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3LOB(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3REF(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CMP3UR(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_CNVMSC(void *ctx, void const *src1, ub1 opc, void *dst);
pevm_excs pevm_CONC3(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_CONC4(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_CONCN(void *ctx, void **args, ub4 nargs, boolean nocheck);
#define    pevm_CVTCD(ctx, src1, dst) pevm_CVTCFD(ctx, src1, (void *)0, dst)
pevm_excs pevm_CVTCFD(void *ctx, void const *src1, void const *src2, 
                      void *dst);
#define pevm_CVTCL(ctx, src1, dst) pevm_CVTCFL(ctx, src1, (void *)0, dst)
pevm_excs pevm_CVTCFL(void *ctx, void const *src1, void const *src2, 
                      void *dst);
#define    pevm_CVTCI(ctx, src1, dst) pevm_CVTCI_i(ctx, src1, dst, TRUE)
#define    pevm_CVTNI(ctx, src1, dst) pevm_CVTCI_i(ctx, src1, dst, FALSE)
pevm_excs pevm_CVTCI_i(void *ctx, void const *src1, void *dst, boolean cvtci);
pevm_excs pevm_CVTCN(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTCUR(void *ctx, void const *src1, void *dst);
#define    pevm_CVTDC(ctx, src1, dst) pevm_CVTDFC(ctx, src1, (void *)0, dst)
pevm_excs pevm_CVTDFC(void *ctx, void const *src1, void const *src2, 
                      void *dst);
pevm_excs pevm_CVTEI(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTHR(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTIC(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTIE(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTIN(void *ctx, void const *src1, void *dst);
#define pevm_CVTLC(ctx, src1, dst) pevm_CVTLFC(ctx, src1, (void *)0, dst)
pevm_excs pevm_CVTLFC(void *ctx, void const *src1, void const *src2, 
                      void *dst);
#define    pevm_CVTNC(ctx, src1, dst) pevm_CVTNFC(ctx, src1, (void *)0, dst)
pevm_excs pevm_CVTNFC(void *ctx, void const *src1, void const *src2, 
                      void *dst);
pevm_excs pevm_CVTRH(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CVTURC(void *ctx, void const *src1, void *dst);
pevm_excs pevm_DIVN(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_EXECC(void *ctx, void const *src1, ub4 tmpub4);
pevm_excs pevm_EXPI(void *ctx, void const *src1, void const *src2, void *dst);
#define   pevm_FTCHC(ctx, src1) pevm_BFTCHC(ctx, src1, (void *)0)
pevm_excs pevm_GRWM(void *ctx, ub1 tmpub1, void const *src, void *dst);

/* dynamic sql: initialize stmt (cursor) for execute immediate */
pevm_excs pevm_I4EXIM(void *ctx, void *stmt, const void *sql_str, ub2 nbinds,
                      ub2 ndefines, ub4 epurity, ub4 opf);

/* dynamic : execute immediate */
pevm_excs pevm_EXIM(void *ctx, void const *src1);

/* initialize for opening a cursor for a dynamic string */
pevm_excs pevm_I4OPND(void *ctx, ub1 regs, sb4 offs, void *stmt,
                      void const *sql_str, ub2 nbinds, ub4 epurity);

/* dynamic sql: do a dynamic OPEN on a query string */
pevm_excs pevm_OPND(void *ctx, void const *src1);

pevm_excs pevm_INITX(void *ctx, void const *src1, const void *src2);
pevm_excs pevm_INSTNL(void *perc1, ub2 did, ub1 reuse, ub1 instkind);
pevm_excs pevm_MODI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_MOVA(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVADT(void *ctx, void const *src1, void *dst);
#define pevm_MOVC(ctx, src1, dst) pevm_MOVC_i(ctx, src1, dst, FALSE)
#define pevm_MOVCB(ctx, src1, dst) pevm_MOVC_i(ctx, src1, dst, TRUE)
pevm_excs pevm_MOVC_i(void *ctx, ub1 opc, void const *src1, 
                    void *dst);
pevm_excs pevm_MOVCR(void *ctx, void const *src1, void const *src2, 
                     ub1 tmpub1, void *dst, ub1 dregs, sb4 doffs);
pevm_excs pevm_MOVD(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVDTM(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVI(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVITV(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVLOB_i(void *ctx, void const *src1, void *dst,
                        boolean shallowcp);
#define pevm_MOVLOB(ctx, src1, dst) pevm_MOVLOB_i(ctx, src1, dst, FALSE)
#define pevm_MOVSHLOB(ctx, src1, dst) pevm_MOVLOB_i(ctx, src1, dst, TRUE)
pevm_excs pevm_MOVN(void *ctx, void const *src1, void const *src2, 
                         void const *src3, void *dst);
pevm_excs pevm_MOVNU(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVOPQ(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVRAW(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVREF(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MOVSELFA(void *ctx, void const *src1, void *dst);
#define pevm_MOVSHADT pevm_MOVADT
pevm_excs pevm_MOVUR(void *ctx, void const *src1, void *dst);
pevm_excs pevm_MULI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_MULN(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_NCAL(void *ctx, ub2 did, ub2 ept, void **arg_block, void *ump);
pevm_excs pevm_SNCAL(void *ctx, ub2 did, ub2 ept, void **arg_block);
pevm_excs pevm_DNCAL(void *ctx, ub2 ept, ub2 vti);
pevm_excs pevm_DCAL(void *ctx, ub2 ept, ub2 vti, ub1 **pc, dvoid ** arg_block);
pevm_excs pevm_DRCAL(void *ctx, ub2 did, ub2 vti, ub2 ept);
pevm_excs pevm_NEGI(void *ctx, void const *src1, void *dst);
pevm_excs pevm_NEGN(void *ctx, void const *src1, void *dst);
pevm_excs pevm_PATXS(void *ctx);
pevm_excs pevm_PIPE(void *ctx, void const *var);
pevm_excs pevm_PRFTC(void *ctx, void const *src1, void const *src2);
pevm_excs pevm_RASIX(void *ctx, ub4 excp);
pevm_excs pevm_RASUX(void *ctx, ub2 did, ub2 idn);
pevm_excs pevm_RCAL(void *ctx, void const *src, void **arg_block);
pevm_excs pevm_REMI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_RLSM(void *ctx);
pevm_excs pevm_SETN(void *ctx, void *dst, ub1 tcat);
pevm_excs pevm_SUBI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_SUBN(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_TSTB(void *ctx, void const *src1);
pevm_excs pevm_TSTC(void *ctx, void const *src1);
pevm_excs pevm_TSTN(void *ctx, void const *src1);
pevm_excs pevm_TSTREF(void *ctx, void const *src1);
pevm_excs pevm_XORI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_INCI(void *ctx, void *src1);                 /* pls_integer++ */
pevm_excs pevm_DECI(void *ctx, void *src1);                 /* pls_integer-- */

/* Frame Initialization */
#define PEVM_INIT_FRAME_HDL(hdl, mptr, dptr) \
      PETmut(hdl) = (void *)mptr;\
      PMUlen(((plsmut*)mptr)) = 0;\
      PMUpd(((plsmut*)mptr)) = (ub1 *)dptr;\
      PMUflg(((plsmut*)mptr)) = PLSFNULL;\
      if (dptr != (void *)0)\
       PMUflg(((plsmut*)mptr)) |= PLSFPA;


/*
 * Scalar's are quite common in generated code.
 * Rather than use the generic PEVM_INIT_FRAM_HDL
 * macro (which results in code bloat and possibly
 * poor optimization of generated code), lets use 
 * special case versions which are optimized for the
 * inlined/out-lined flavors of scalars.
 */
/* initialize Inlined Simple SCALAR */
#define pevm_INSI_ISSCALAR(hdl, mptr, dptr) \
      PETmut(hdl) = (void *)mptr;\
      PMUlen(((plsmut*)mptr)) = 0;\
      PMUpd(((plsmut*)mptr)) = (ub1 *)dptr;\
      PMUflg(((plsmut*)mptr)) = PLSFNULL | PLSFPA;

/* initialize Outlined Simple SCALAR */
#define pevm_INSI_OSSCALAR(hdl, mptr) \
      PETmut(hdl) = (void *)mptr;\
      PMUlen(((plsmut*)mptr)) = 0;\
      PMUpd(((plsmut*)mptr)) = (ub1 *)0;\
      PMUflg(((plsmut*)mptr)) = PLSFNULL;


#define pevm_INSI_CURSREF(ctx, hdl, mptr, dptr) \
  PEVM_INIT_FRAME_HDL(hdl, mptr, dptr)

pevm_excs pevm_INSI_UROWID(void *ctx, void *hdl, plsmut *mptr,  void *dptr,
                           ub4 maxlen, ub1 csform, ub1 lensem, 
                           pemttcat tcat, ub1 regs);

pevm_excs pevm_INSI_CHAR(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                         ub4 maxlen, ub1 csform, ub1 lensem,
                         pemttcat tcat, ub1 regs);

pevm_excs pevm_INSI_LOB(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub1 lobtype, ub1 csform,
                        ub1 regs);

pevm_excs pevm_INSI_DATETIME(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                             ub1 dtmtyp, ub1 prec, ub1 regs);

pevm_excs pevm_INSI_INTERVAL(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                             ub1 inttyp, ub1 ldp, ub1 tlp, ub1 regs);

pevm_excs pevm_INSI_ADT(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub4 ltdo_off, ub1 nact, ub1 regs);

pevm_excs pevm_INSI_OPQ(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub4 ltdo_off, ub1 regs);

pevm_excs pevm_INSI_OBJREF(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                           ub4 ltdo_off, ub1 regs);

pevm_excs pevm_INSI_INDEXED_SSCALAR(void *ctx, void *hdl, plsmut *mptr, 
                                    void *dptr, void *ehdl, ub4 ltdo_off, 
                                    ub1 eltcat, ub1 regs);

pevm_excs pevm_INSI_INDEXED_CHAR(void *ctx, void *hdl, plsmut *mptr, 
                                 void *dptr, void *ehdl, ub4 ltdo_off, 
                                 ub1 eltcat, ub1 regs,
                                 ub1 csf, ub4 mxl, ub1 lensem);

pevm_excs pevm_INSI_INDEXED_LOB(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                void *ehdl, ub4 ltdo_off, ub1 eltcat, ub1 regs,
                                ub1 csf, ub1 lobtype);

pevm_excs pevm_INSI_INDEXED_DATETIME(void *ctx, void *hdl, plsmut *mptr, 
                                     void *dptr, void *ehdl, ub4 ltdo_off, 
                                     ub1 eltcat, ub1 regs,
                                     ub1 dty, ub1 fsprec);

pevm_excs pevm_INSI_INDEXED_INTERVAL(void *ctx, void *hdl, plsmut *mptr, 
                                     void *dptr,
                                     void *ehdl, ub4 ltdo_off, ub1 eltcat, 
                                     ub1 regs,
                                     ub1 dty, ub1 ldp, ub1 fsprec);

pevm_excs pevm_INSI_INDEXED_ADT(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                void *ehdl, ub4 ltdo_off, ub1 eltcat, ub1 regs,
                                ub4 eltdo, ub1 nact);

pevm_excs pevm_INSI_INDEXED_OPQ(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                void *ehdl, ub4 ltdo_off, ub1 eltcat, ub1 regs,
                                ub4 eltdo);

pevm_excs pevm_INSI_INDEXED_OBJREF(void *ctx, void *hdl, plsmut *mptr, 
                                   void *dptr, void *ehdl, ub4 ltdo_off, 
                                   ub1 eltcat, ub1 regs, ub4 eltdo);

pevm_excs pevm_INSI_INDEXED_INDEXED(void *ctx, void *hdl, plsmut *mptr, 
                                    void *dptr, void *ehdl, ub4 ltdo_off, 
                                    ub1 eltcat, ub1 regs, ub4 eltdo_off);


#define pevm_INBI_ISSCALAR(hdl, mptr, dptr) \
        pevm_INSI_ISSCALAR(hdl, mptr, dptr)

#define pevm_INBI_OSSCALAR(hdl, mptr) \
        pevm_INSI_OSSCALAR(hdl, mptr)

pevm_excs pevm_INBI_CURSREF(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                            ub1 tcatval);

pevm_excs pevm_INBI_UROWID(void *ctx, void *hdl, plsmut *mptr,  void *dptr,
                           ub2 bind_num, ub1 csform, ub1 lensem, 
                           pemttcat tcat);

pevm_excs pevm_INBI_CHAR(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                         ub4 bind_num, ub1 csform, ub1 lensem,
                         pemttcat tcat, ub1 regs);

pevm_excs pevm_INBI_LOB(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub2 bind_num, ub1 lobtype, ub1 csform,
                        ub1 regs);

pevm_excs pevm_INBI_DATETIME(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                             ub2 bind_num, ub1 dtmtyp, ub1 prec, ub1 regs);

pevm_excs pevm_INBI_INTERVAL(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                             ub2 bind_num, ub1 inttyp, ub1 ldp, ub1 tlp, 
                             ub1 regs);

pevm_excs pevm_INBI_ADT(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub4 ltdo_off, ub2 bind_num, ub1 nact, ub1 regs);

pevm_excs pevm_INBI_OPQ(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                        ub4 ltdo_off,  ub2 bind_num, ub1 regs);

pevm_excs pevm_INBI_OBJREF(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                           ub4 ltdo_off, ub2 bind_num, ub1 regs);

pevm_excs pevm_INBI_INDEXED_SSCALAR(void *ctx, void *hdl, plsmut *mptr, 
                                    void *dptr,
                                    void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                    ub1 eltcat, ub1 regs);

pevm_excs pevm_INBI_INDEXED_UROWID(void *ctx, void *hdl, plsmut *mptr, 
                                   void *dptr,
                                   void *ehdl, ub2 bind_num, 
                                   ub4 ltdo_off, ub1 eltcat,
                                   ub1 regs, ub1 csf,
                                   ub4 mxl, 
                                   ub1 lensem);

pevm_excs pevm_INBI_INDEXED_CHAR(void *ctx, void *hdl, plsmut *mptr, 
                                 void *dptr,
                                 void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                 ub1 eltcat, ub1 regs,
                                 ub1 csf, ub4 mxl, ub1 lensem);

pevm_excs pevm_INBI_INDEXED_DATETIME(void *ctx, void *hdl, plsmut *mptr, 
                                     void *dptr,
                                     void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                     ub1 eltcat, ub1 regs,
                                     ub1 dty, ub1 prec);

pevm_excs pevm_INBI_INDEXED_INTERVAL(void *ctx, void *hdl, plsmut *mptr, 
                                     void *dptr,
                                     void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                     ub1 eltcat, ub1 regs,
                                     ub1 dty, ub1 ldp, ub1 prec);

pevm_excs pevm_INBI_INDEXED_LOB(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                ub1 eltcat, ub1 regs,
                                ub1 csf, ub1 lobtyp);

pevm_excs pevm_INBI_INDEXED_ADT(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                 void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                 ub1 eltcat, ub1 regs,
                                 ub4 eltdo, ub1 nact);

pevm_excs pevm_INBI_INDEXED_OPQ(void *ctx, void *hdl, plsmut *mptr, void *dptr,
                                void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                ub1 eltcat, ub1 regs,
                                ub4 eltdo);

pevm_excs pevm_INBI_INDEXED_OBJREF(void *ctx, void *hdl, plsmut *mptr, 
                                   void *dptr,
                                   void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                   ub1 eltcat, ub1 regs,
                                   ub4 eltdo);

pevm_excs pevm_INBI_INDEXED_INDEXED(void *ctx, void *hdl, plsmut *mptr, 
                                    void *dptr,
                                    void *ehdl, ub2 bind_num, ub4 ltdo_off, 
                                    ub1 eltcat, ub1 regs,  ub4 eltdo_off
                                    );

pevm_excs pevm_SCNST(void *ctx, ub4 tmpub4, void *dst);
pevm_excs pevm_CCNST(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CCSINF(void *ctx, void const *src1, void *dst, ub1 srctc, 
                      ub1 dsttc);
pevm_excs pevm_EXCOD(void *ctx, void *dst);
pevm_excs pevm_EXMSG(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INSTC2(void *ctx, void const *src1, void const *src2, 
                      void *dst, ub1 regs, sb4 offs, boolean instc2);
pevm_excs pevm_SCNST(void *ctx, ub4 tmpub4, void *dst);
pevm_excs pevm_CCSINF(void *ctx, void const *src1, void *dst, ub1 srctc, 
                      ub1 dsttc);
pevm_excs pevm_EXCOD(void *ctx, void *dst);
pevm_excs pevm_EXMSG(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CLOSC(void *ctx, void const *src1, ub1 suppress_error);
pevm_excs pevm_BIND(void *ctx, void const *src1, ub2 position, ub2 tmpub2,
                    void const *src2, ub2 flags, void const *src3, 
                    ub2 attr_no, ub1 opc);
pevm_excs pevm_DEFINE(void *ctx, void const *src1, ub2 position, ub2 tmpub2,
                      void *src2);
pevm_excs pevm_FCAL(void *ctx, void const *src1);
pevm_excs pevm_ADEFINE(void *ctx, void const *src1, ub2 position, ub2 tmpub2,
                       ub1 tmpub1, void const *src2, ub2 bind_num, 
                       void const *src3, void const *src4, 
                       ub2 attr_no, ub1 opc);
pevm_excs pevm_BDCINI(void *ctx, void const *src1, ub1 bdflags);
pevm_excs pevm_PUTX(void *ctx, void const *src1, void const *src2, 
                    void *dst);
pevm_excs pevm_BCNSTR(void *ctx, void const *src1, sb4 prec, sb4 scale, 
                      ub2 position, ub4 tmpub4);
pevm_excs pevm_GRWFCC_c(void *ctx, void const *src1, ub4 tmpub4, ub1 csform,
                        void *dst);
pevm_excs pevm_GRWFCC(void *ctx, void const *src1, ub1 csform, void *dst);

pevm_excs pevm_GRWFCL(void *ctx, ub4 tmpub4, ub1 csform, void *dst);

pevm_excs pevm_RET(void *ctx, ub1 **pc);
#define pevm_RNDD(ctx, src1, dst) pevm_RNDDC_i(ctx, src1, (void *)0, dst, TRUE)
#define pevm_RNDDC(ctx, src1, src2, dst) \
  pevm_RNDDC_i(ctx, src1, src2, dst, TRUE)
#define pevm_TRND(ctx, src1, dst) \
  pevm_RNDDC_i(ctx, src1, (void *)0, dst, FALSE)
#define pevm_TRNDC(ctx, src1, src2, dst) \
  pevm_RNDDC_i(ctx, src1, src2, dst, FALSE)

pevm_excs pevm_RNDDC_i(void *ctx, void const *src1, void const *src2, 
                       void *dst, boolean rnddp);
pevm_excs pevm_LSTD(void *ctx, void const *src1, void *dst);

#define pevm_ADDDN(ctx, src1, src2, dst) \
  pevm_ADDDN_i(ctx, src1, src2, dst, FALSE)
#define pevm_SUBDN(ctx, src1, src2, dst) \
  pevm_ADDDN_i(ctx, src1, src2, dst, TRUE)
pevm_excs pevm_ADDDN_i(void *ctx, void const *src1, void const *src2, 
                       void *dst, boolean subdnp);
pevm_excs pevm_SUBDD(void *ctx, void const *src1, void const *src2, 
                     void *dst);
pevm_excs pevm_ADDMDN(void *ctx, void const *src1, void const *src2, 
                      void *dst);
pevm_excs pevm_MBTD(void *ctx, void const *src1, void const *src2, 
                    void *dst);
pevm_excs pevm_NXTD(void *ctx, void const *src1, void const *src2, 
                    void *dst);

pevm_excs pevm_ENTER(void *ctx, sb2 scope, ub4 tmpub4, ub1 **pc,
                     void *frame, ub1 ***ppreg, ub4 **lnr, 
                     ub1 *state_buf, size_t state_buf_size);

pevm_excs pevm_BNDS(void *ctx, void *bnds_curs, void const *src1, 
                    void const *src2);
pevm_excs pevm_COPN(void *ctx, void const *src1);
pevm_excs pevm_icd_call_common(void *ctx, ub2 did, ub2 ept, ub2 loc, ub2 argc,
                               boolean is_std, void **arg_block);
pevm_excs pevm_GBCR(void *ctx, ub2 tmpub2, void const *src1, void *dst);
pevm_excs pevm_CFND(void *ctx, void const *src1);
pevm_excs pevm_CSFND(void *ctx);
pevm_excs pevm_CRWC(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CSRWC(void *ctx, void *dst);
pevm_excs pevm_BCRWC(void *ctx, void const *src1, void *dst, void const *src2);
pevm_excs pevm_BCSRWC(void *ctx, void *dst, void const *src1);
pevm_excs pevm_GBVAR(void *ctx, ub2 tmpub2, ub2 bind_num, void *dst);
pevm_excs pevm_SBVAR(void *ctx, ub2 tmpub2, ub2 bind_num, const void *src1);
pevm_excs pevm_GBEX(void *ctx, ub2 tmpub2, ub2 bind_num, 
                    void const *src1, void *dst);
pevm_excs pevm_SBEX(void *ctx, ub2 tmpub2, ub2 bind_num, 
                    void const *src1, void const *src2);
pevm_excs pevm_GETFX(void *ctx, ub2 tmpub2, void *dst, ub2 eltty);
pevm_excs pevm_SETFX(void *ctx, ub2 tmpub2, void const *src1, ub2 eltty);
pevm_excs pevm_DRPX(void *ctx, void const *src1);
pevm_excs pevm_GETX(void *ctx, void const *src1, void const *src2, 
                    void *dst);
pevm_excs pevm_MOVX(void *ctx, void const *src1, void *dst);
pevm_excs pevm_EXTX(void *ctx, void const *src1, ub4 tmpub4);

pevm_excs pevm_INMDH_CHAR(void *ctx, void *src1, ub1 csform, ub4 ctmxlen, 
                          ub1 lensem, ub1 tcat);

pevm_excs pevm_INMDH_LOB(void *ctx, void *src1, ub1 csform, ub1 lobtyp);

pevm_excs pevm_INMDH_DATETIME(void *ctx, void *src1, ub1 typ, ub1 tlp);

pevm_excs pevm_INMDH_INTERVAL(void *ctx, void *src1, ub1 typ, ub1 ldp, 
                              ub1 tlp);

pevm_excs pevm_INMDH_ADT(void *ctx, void  *src1, ub4 chtdo);
pevm_excs pevm_INMDH_INDEXED_SSCALAR(void *ctx, void  *src1, ub4 chtdo, 
                                     ub1 etcat, void *ehdl);
pevm_excs pevm_INMDH_INDEXED_OBJREF(void *ctx, void  *src1, ub4 chtdo, 
                                    ub1 etcat, void *ehdl, ub4 eltdo);
pevm_excs pevm_INMDH_INDEXED_OPQ(void *ctx, void  *src1, ub4 chtdo, 
                                 ub1 etcat, void *ehdl, ub4 eltdo);
pevm_excs pevm_INMDH_INDEXED_INDEXED(void *ctx, void  *src1, ub4 chtdo, 
                                     ub1 etcat, void *ehdl, ub4 eltdo);
pevm_excs pevm_INMDH_INDEXED_ADT(void *ctx, void  *src1, ub4 chtdo, 
                                 ub1 etcat, void *ehdl, ub4 eltdo);
pevm_excs pevm_INMDH_INDEXED_CHAR(void *ctx, void  *src1, ub4 chtdo, 
                                  ub1 etcat, void *ehdl, ub1 csf, 
                                  ub4 maxlen, ub1 lensem);
pevm_excs pevm_INMDH_INDEXED_UROWID(void *ctx, void  *src1, ub4 chtdo, 
                                    ub1 etcat, void *ehdl,  ub1 csf,
                                    ub4 maxlen, ub1 lensem);
pevm_excs pevm_INMDH_INDEXED_LOB(void *ctx, void  *src1, ub4 chtdo, 
                                 ub1 etcat, void *ehdl, ub1 csf, 
                                 ub1 maxlen);
pevm_excs pevm_INMDH_INDEXED_DATETIME(void *ctx, void  *src1, ub4 chtdo, 
                                      ub1 etcat, void *ehdl,
                                      ub1 sqlt, ub1 fsprec);
pevm_excs pevm_INMDH_INDEXED_INTERVAL(void *ctx, void  *src1, ub4 chtdo, 
                                      ub1 etcat, void *ehdl,
                                      ub1 sqlt, ub1 ldp, ub1 fsprec);
pevm_excs pevm_INMDH_OPQ(void *ctx, void  *src1, ub4 ophtdo);
pevm_excs pevm_INMDH_OBJREF(void *ctx, void  *src1, ub4 orhtdo);

pevm_excs pevm_INHFA_COMMON(void *ctx, void const *src1, void *dst, 
                             ub4 index_path_len, ...);

pevm_excs pevm_INHFA_CHAR(void *ctx, void const *src1, void *dst, ub1 csform,
                         ub4 ctmxlen, ub1 lensem, ub1 tcat);

pevm_excs pevm_INHFA_FCHAR(void *ctx, void const *src1, void *dst, ub1 csform, 
                         ub4 ctmxlen, ub1 lensem, ub1 tcat);

pevm_excs pevm_INHFA_LOB(void *ctx, void const *src1, void *dst, ub1 lobtyp, 
                         ub1 csform);

pevm_excs pevm_INHFA_OBJREF(void *ctx, void const *src1, void *dst, 
                         ub4 ltdo_off);

pevm_excs pevm_INHFA_DATETIME(void *ctx, void const *src1, void *dst, ub1 typ,
                         ub1 tlp);

pevm_excs pevm_INHFA_INTERVAL(void *ctx, void const *src1, void *dst, ub1 typ,
                         ub1 tlp, ub1 ldp);

pevm_excs pevm_INHFA_ADT(void *ctx, void const *src1, void *dst, ub4 chtdo);

pevm_excs pevm_INHFA_OPQ(void *ctx, void const *src1, void *dst, ub4 ophtdo);

pevm_excs pevm_INHFA_INDEXED_SSCALAR(void *ctx, void const *src1, void *dst, 
                                     ub4 tdo_off, void *ehdl, ub1 eltcat);

pevm_excs pevm_INHFA_INDEXED_CHAR(void *ctx, void const *src1, void *dst, 
                                  ub4 tdo_off, void *ehdl, ub1 eltcat,
                                  ub1 csf, ub4 mxln, ub1 lensem);

pevm_excs pevm_INHFA_INDEXED_LOB(void *ctx, void const *src1, void *dst, 
                                 ub4 tdo_off, void *ehdl, ub1 eltcat,
                                 ub1 lobtyp, ub1 csf, ub1 lensem);

pevm_excs pevm_INHFA_INDEXED_DATETIME(void *ctx, void const *src1, void *dst, 
                                      ub4 tdo_off, void *ehdl, ub1 eltcat,
                                      ub1 typ, ub1 tlp);

pevm_excs pevm_INHFA_INDEXED_INTERVAL(void *ctx, void const *src1, void *dst, 
                                      ub4 tdo_off, void *ehdl, ub1 eltcat,
                                      ub1 typ, ub1 ldp, ub1 tlp);

pevm_excs pevm_INHFA_INDEXED_ADT(void *ctx, void const *src1, void *dst, 
                                 ub4 tdo_off, void *ehdl, ub1 eltcat,
                                 ub4 eltdo_off);

pevm_excs pevm_INHFA_INDEXED_INDEXED(void *ctx, void const *src1, void *dst, 
                                     ub4 tdo_off, void *ehdl, ub1 eltcat,
                                     ub4 eltdo_off);


pevm_excs pevm_INHFA_INDEXED_OPQ(void *ctx, void const *src1, void *dst, 
                                 ub4 tdo_off, void *ehdl, ub1 eltcat,
                                 ub4 eltdo_off);

pevm_excs pevm_INHFA_INDEXED_OBJREF(void *ctx, void const *src1, void *dst, 
                                    ub4 tdo_off, void *ehdl, ub1 eltcat,
                                    ub4 eltdo_off);

pevm_excs pevm_INRDH(void *ctx, void const *src1, void *dst,
                            pemttcat tcat, ub4 index_path_len, ...);

pevm_excs pevm_INRDH_common(void *ctx, void *dst, void const *src1, 
                                   pemttcat eltcat, pemttcat src1tcat);


pevm_excs pevm_INRDH_CHAR(void *ctx, void const *src1, void *dst);

pevm_excs pevm_INRDH_LOB(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INRDH_DATETIME(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INRDH_INTERVAL(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INRDH_ADT(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INRDH_OBJREF(void *ctx, void const *src1, void *dst);
pevm_excs pevm_INRDH_INDEXED(void *ctx, void const *src1, void *dst);
pevm_excs pevm_TREAT(void *ctx, 
                     void const *src1, ub4 src2, ub1 isref, void *dst);
pevm_excs pevm_CMPIO(void *ctx, 
                     void const *src1, ub4 src2, ub1 flag);

pevm_excs pevm_ABSN(void *ctx, void const *src1, void *dst);
pevm_excs pevm_DIVI(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_NULCHK(void *ctx, const ub1 nul_code, void const* src1);
pevm_excs pevm_RNGCHKI(void *ctx, const ub1 range_code,
                       void const* src1, void const* bndl, void const* bndh);
pevm_excs pevm_RNGCHKF(void *ctx, const ub1 range_code,
                       void const* src1, void const* bndl, void const* bndh);
pevm_excs pevm_ANDB(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_ORB(void *ctx, void const *src1, void const *src2, void *dst);
pevm_excs pevm_NOTB(void *ctx, void const *src1, void *dst);
pevm_excs pevm_CHSNULL(void *ctx, const ub1 chs_code, 
                       void const *src1, void const *src2, void const *src3, 
                       void *dst);
pevm_excs pevm_REL2BOOL(void *ctx, const ub1 rel_code, 
                        void const *src1, void const *src2, void *dst);
pevm_excs pevm_MINMAX(void *ctx, const ub1 ext_code, 
                      void const *src1, void const *src2, void *dst);

                          
/*---------------------------------------------------------------------------
                          PRIVATE FUNCTIONS
 ---------------------------------------------------------------------------*/


/* DON'T ADD ANY PRIVATE FUNCTIONS TO THIS FILE. THIS FILE IS SHIPPED
 * FOR PL/SQL NCOMP. ADD PRIVATE FUNCTIONS TO pvm0.h.
 */


#endif                                              /* PEVM_ORACLE */
